@@ -1,5 +1,5 @@
 package Consul::API::Agent;
-$Consul::API::Agent::VERSION = '0.004';
+$Consul::API::Agent::VERSION = '0.005';
 use namespace::autoclean;
 
 use Moo::Role;
@@ -27,22 +27,30 @@ use Carp qw(croak);
 
 sub checks {
     my ($self, %args) = @_;
-    [ map { Consul::API::Agent::Check->new(%$_) } values %{$$self->_api_exec($$self->_agent_endpoint."/checks", 'GET', %args)} ];
+    $$self->_api_exec($$self->_agent_endpoint."/checks", 'GET', %args, sub {
+        [ map { Consul::API::Agent::Check->new(%$_) } values %{$_[0]} ]
+    });
 }
 
 sub services {
     my ($self, %args) = @_;
-    [ map { Consul::API::Agent::Service->new(%$_) } values %{$$self->_api_exec($$self->_agent_endpoint."/services", 'GET', %args)} ];
+    $$self->_api_exec($$self->_agent_endpoint."/services", 'GET', %args, sub {
+        [ map { Consul::API::Agent::Service->new(%$_) } values %{$_[0]} ]
+    });
 }
 
 sub members {
     my ($self, %args) = @_;
-    [ map { Consul::API::Agent::Member->new(%$_) } @{$$self->_api_exec($$self->_agent_endpoint."/members", 'GET', %args)} ];
+    $$self->_api_exec($$self->_agent_endpoint."/members", 'GET', %args, sub {
+        [ map { Consul::API::Agent::Member->new(%$_) } @{$_[0]} ]
+    });
 }
 
 sub self {
     my ($self, %args) = @_;
-    Consul::API::Agent::Self->new(%{$$self->_api_exec($$self->_agent_endpoint."/self", 'GET', %args)});
+    $$self->_api_exec($$self->_agent_endpoint."/self", 'GET', %args, sub {
+        Consul::API::Agent::Self->new(%{$_[0]})
+    });
 }
 
 sub maintenance {
@@ -123,7 +131,7 @@ sub service_maintenance {
 }
 
 package Consul::API::Agent::Check;
-$Consul::API::Agent::Check::VERSION = '0.004';
+$Consul::API::Agent::Check::VERSION = '0.005';
 use Moo;
 use Types::Standard qw(Str);
 
@@ -137,7 +145,7 @@ has service_id   => ( is => 'ro', isa => Str, init_arg => 'ServiceID',   require
 has service_name => ( is => 'ro', isa => Str, init_arg => 'ServiceName', required => 1 );
 
 package Consul::API::Agent::Service;
-$Consul::API::Agent::Service::VERSION = '0.004';
+$Consul::API::Agent::Service::VERSION = '0.005';
 use Moo;
 use Types::Standard qw(Str Int ArrayRef);
 
@@ -148,7 +156,7 @@ has port    => ( is => 'ro', isa => Int,           init_arg => 'Port',    requir
 has tags    => ( is => 'ro', isa => ArrayRef[Str], init_arg => 'Tags',    required => 1, coerce => sub { $_[0] || [] } );
 
 package Consul::API::Agent::Member;
-$Consul::API::Agent::Member::VERSION = '0.004';
+$Consul::API::Agent::Member::VERSION = '0.005';
 use Moo;
 use Types::Standard qw(Str Int HashRef);
 
@@ -165,7 +173,7 @@ has delegate_max => ( is => 'ro', isa => Int,          init_arg => 'DelegateMax'
 has delegate_cur => ( is => 'ro', isa => Int,          init_arg => 'DelegateCur', required => 1 );
 
 package Consul::API::Agent::Self;
-$Consul::API::Agent::Self::VERSION = '0.004';
+$Consul::API::Agent::Self::VERSION = '0.005';
 use Moo;
 use Types::Standard qw(HashRef);
 use Type::Utils qw(class_type);
