@@ -1,5 +1,5 @@
 package Consul::API::Catalog;
-$Consul::API::Catalog::VERSION = '0.004';
+$Consul::API::Catalog::VERSION = '0.005';
 use namespace::autoclean;
 
 use Moo::Role;
@@ -32,7 +32,9 @@ sub datacenters {
 
 sub nodes {
     my ($self, %args) = @_;
-    [ map { Consul::API::Catalog::ShortNode->new(%$_) } @{$$self->_api_exec($$self->_catalog_endpoint."/nodes", 'GET', %args)} ];
+    $$self->_api_exec($$self->_catalog_endpoint."/nodes", 'GET', %args, sub {
+        [ map { Consul::API::Catalog::ShortNode->new(%$_) } @{$_[0]} ]
+    });
 }
 
 sub services {
@@ -43,7 +45,9 @@ sub services {
 sub service {
     my ($self, $service, %args) = @_;
     croak 'usage: $catalog->service($service, [%args])' if grep { !defined } ($service);
-    [ map { Consul::API::Catalog::Service->new(%$_) } @{$$self->_api_exec($$self->_catalog_endpoint."/service/".$service, 'GET', %args)} ];
+    $$self->_api_exec($$self->_catalog_endpoint."/service/".$service, 'GET', %args, sub {
+        [ map { Consul::API::Catalog::Service->new(%$_) } @{$_[0]} ]
+    });
 }
 
 sub register {
@@ -59,11 +63,13 @@ sub deregister {
 sub node {
     my ($self, $node, %args) = @_;
     croak 'usage: $catalog->node($node, [%args])' if grep { !defined } ($node);
-    Consul::API::Catalog::Node->new($$self->_api_exec($$self->_catalog_endpoint."/node/".$node, 'GET', %args));
+    $$self->_api_exec($$self->_catalog_endpoint."/node/".$node, 'GET', %args, sub {
+        Consul::API::Catalog::Node->new($_[0])
+    });
 }
 
 package Consul::API::Catalog::ShortNode;
-$Consul::API::Catalog::ShortNode::VERSION = '0.004';
+$Consul::API::Catalog::ShortNode::VERSION = '0.005';
 use Moo;
 use Types::Standard qw(Str);
 
@@ -71,7 +77,7 @@ has name    => ( is => 'ro', isa => Str, init_arg => 'Node',    required => 1 );
 has address => ( is => 'ro', isa => Str, init_arg => 'Address', required => 1 );
 
 package Consul::API::Catalog::Service;
-$Consul::API::Catalog::Service::VERSION = '0.004';
+$Consul::API::Catalog::Service::VERSION = '0.005';
 use Moo;
 use Types::Standard qw(Str Int ArrayRef);
 
@@ -83,7 +89,7 @@ has address => ( is => 'ro', isa => Str,           init_arg => 'Address',     re
 has tags    => ( is => 'ro', isa => ArrayRef[Str], init_arg => 'ServiceTags', required => 1, coerce => sub { $_[0] || [] } );
 
 package Consul::API::Catalog::Node;
-$Consul::API::Catalog::Node::VERSION = '0.004';
+$Consul::API::Catalog::Node::VERSION = '0.005';
 use Moo;
 use Types::Standard qw(HashRef);
 use Type::Utils qw(class_type);
