@@ -1,5 +1,5 @@
 package Consul::API::Event;
-$Consul::API::Event::VERSION = '0.004';
+$Consul::API::Event::VERSION = '0.005';
 use namespace::autoclean;
 
 use Moo::Role;
@@ -34,11 +34,13 @@ sub fire {
 
 sub list {
     my ($self, %args) = @_;
-    [ map { Consul::API::Event::Event->new(%$_) } @{$$self->_api_exec($$self->_event_endpoint."/list", 'GET', %args)} ];
+    $$self->_api_exec($$self->_event_endpoint."/list", 'GET', %args, sub {
+        [ map { Consul::API::Event::Event->new(%$_) } @{$_[0]} ]
+    });
 }
 
 package Consul::API::Event::Event;
-$Consul::API::Event::Event::VERSION = '0.004';
+$Consul::API::Event::Event::VERSION = '0.005';
 use Convert::Base64 qw(decode_base64);
 
 use Moo;
