@@ -1,5 +1,5 @@
 package Consul::API::Health;
-$Consul::API::Health::VERSION = '0.004';
+$Consul::API::Health::VERSION = '0.005';
 use namespace::autoclean;
 
 use Moo::Role;
@@ -28,29 +28,37 @@ use Carp qw(croak);
 sub node {
     my ($self, $node, %args) = @_;
     croak 'usage: $health->node($node, [%args])' if grep { !defined } ($node);
-    [ map { Consul::API::Health::Check->new(%$_) } @{$$self->_api_exec($$self->_health_endpoint."/node/".$node, 'GET', %args)} ];
+    $$self->_api_exec($$self->_health_endpoint."/node/".$node, 'GET', %args, sub {
+        [ map { Consul::API::Health::Check->new(%$_) } @{$_[0]} ]
+    });
 }
 
 sub checks {
     my ($self, $service, %args) = @_;
     croak 'usage: $health->checks($service, [%args])' if grep { !defined } ($service);
-    [ map { Consul::API::Health::Check->new(%$_) } @{$$self->_api_exec($$self->_health_endpoint."/checks/".$service, 'GET', %args)} ];
+    $$self->_api_exec($$self->_health_endpoint."/checks/".$service, 'GET', %args, sub {
+        [ map { Consul::API::Health::Check->new(%$_) } @{$_[0]} ]
+    });
 }
 
 sub service {
     my ($self, $service, %args) = @_;
     croak 'usage: $health->service($service, [%args])' if grep { !defined } ($service);
-    [ map { Consul::API::Health::Service->new(%$_) } @{$$self->_api_exec($$self->_health_endpoint."/service/".$service, 'GET', %args)} ];
+    $$self->_api_exec($$self->_health_endpoint."/service/".$service, 'GET', %args, sub {
+        [ map { Consul::API::Health::Service->new(%$_) } @{$_[0]} ]
+    });
 }
 
 sub state {
     my ($self, $state, %args) = @_;
     croak 'usage: $health->state($state, [%args])' if grep { !defined } ($state);
-    [ map { Consul::API::Health::Check->new(%$_) } @{$$self->_api_exec($$self->_health_endpoint."/state/".$state, 'GET', %args)} ];
+    $$self->_api_exec($$self->_health_endpoint."/state/".$state, 'GET', %args, sub {
+        [ map { Consul::API::Health::Check->new(%$_) } @{$_[0]} ]
+    });
 }
 
 package Consul::API::Health::Check;
-$Consul::API::Health::Check::VERSION = '0.004';
+$Consul::API::Health::Check::VERSION = '0.005';
 use Moo;
 use Types::Standard qw(Str);
 
@@ -64,7 +72,7 @@ has service_id   => ( is => 'ro', isa => Str, init_arg => 'ServiceID',   require
 has service_name => ( is => 'ro', isa => Str, init_arg => 'ServiceName', required => 1 );
 
 package Consul::API::Health::Service;
-$Consul::API::Health::Service::VERSION = '0.004';
+$Consul::API::Health::Service::VERSION = '0.005';
 use Moo;
 use Types::Standard qw(ArrayRef);
 use Type::Utils qw(class_type);
