@@ -1,5 +1,5 @@
 package Consul::API::KV;
-$Consul::API::KV::VERSION = '0.004';
+$Consul::API::KV::VERSION = '0.005';
 use namespace::autoclean;
 
 use Moo::Role;
@@ -28,7 +28,9 @@ use Carp qw(croak);
 sub get {
     my ($self, $key, %args) = @_;
     croak 'usage: $kv->get($key, [%args])' if grep { !defined } ($key);
-    Consul::API::KV::Response->new($$self->_api_exec($$self->_kv_endpoint."/".$key, 'GET', %args)->[0]);
+    $$self->_api_exec($$self->_kv_endpoint."/".$key, 'GET', %args, sub {
+        Consul::API::KV::Response->new($_[0]->[0]);
+    });
 }
 
 sub put {
@@ -51,7 +53,7 @@ sub keys {
 }
 
 package Consul::API::KV::Response;
-$Consul::API::KV::Response::VERSION = '0.004';
+$Consul::API::KV::Response::VERSION = '0.005';
 use Convert::Base64 qw(decode_base64);
 
 use Moo;
